@@ -7,9 +7,10 @@
 #'
 #' @param B Matrix: Bipartite network
 #'
-#' @return list(positive, negative).
+#' @return list(positive, negative, summary).
 #' positive gives matrix of probability of ties above the observed value.
 #' negative gives matrix of probability of ties below the observed value.
+#' summary: a data frame summary of the inputted matrix and the model used including: model name, number of rows, skew of row sums, number of columns, skew of column sums, and running time.
 #'
 #' @references \href{https://doi.org/10.1007/s13278-013-0107-y}{Neal, Zachary. 2013. “Identifying Statistically Significant Edges in One-Mode Projections.” Social Network Analysis and Mining 3 (4). Springer: 915–24. DOI:10.1007/s13278-013-0107-y.}
 #' @export
@@ -19,19 +20,29 @@
 
 hyperg <- function(B){
 
+  #Run Time
+  run.time.start <- Sys.time()
+
   #Argument Checks
-  if (class(B)!="matrix") {stop("input bipartite data must be a matrix")}
+  if (!(methods::is(B, "matrix")) & !(methods::is(B, "sparseMatrix"))) {stop("input bipartite data must be a matrix")}
   message("Finding the Backbone using Hypergeometric Distribution")
 
-  P <-B%*%t(B)
+  if (methods::is(B, "sparseMatrix")) {
+    P <- Matrix::tcrossprod(B)
+    rs <- Matrix::rowSums(B)
+  } else {
+    P <- tcrossprod(B)
+    rs <- rowSums(B)
+  }
+
   df <- data.frame(as.vector(P))
   names(df)[names(df)=="as.vector.P."] <- "projvalue"
 
   #Compute row sums
-  df$row_sum_i <- rep(rowSums(B), times = nrow(B))
+  df$row_sum_i <- rep(rs, times = nrow(B))
 
   #Match each row sum i with each row sum j and their Pij value
-  df$row_sum_j <- rep(rowSums(B), each = nrow(B))
+  df$row_sum_j <- rep(rs, each = nrow(B))
 
   #Compute different in number of artifacts and row sum
   df$diff <- ncol(B)-df$row_sum_i
@@ -51,5 +62,23 @@ hyperg <- function(B){
   rownames(Negative) <- rownames(B)
   colnames(Negative) <- rownames(B)
 
-  return(list(positive = Positive, negative = Negative))
+  #Run Time
+  run.time.end <- Sys.time()
+  total.time = (round(difftime(run.time.end, run.time.start), 2))
+
+  #Compile Summary
+  if (methods::is(B, "sparseMatrix")) {
+    r <- Matrix::rowSums(B)
+    c <- Matrix::colSums(B)
+  } else {
+    r <- rowSums(B)
+    c <- colSums(B)
+  }
+  a <- c("Model", "Number of Rows", "Skew of Row Sums", "Number of Columns", "Skew of Column Sums", "Running Time")
+  b <- c("Hypergeometric Model", dim(B)[1], round((sum((r-mean(r))**3))/((length(r))*((stats::sd(r))**3)), 5), dim(B)[2], round((sum((c-mean(c))**3))/((length(c))*((stats::sd(c))**3)), 5), as.numeric(total.time))
+  model.summary <- data.frame(a,b, row.names = 1)
+  colnames(model.summary)<-"Model Summary"
+
+  return(list(positive = Positive, negative = Negative, summary = model.summary))
 }
+
